
/********************************************************************
* NAME:                                                             *
*                                                                   *
* PURPOSE:                                                          *
*                                                                   *
* GLOBAL VARIABLES:                                                 *
*                                                                   *
* Variable:                                                         *
*                                                                   *
* Type:                                                             *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* DEVELOPMENT HISTORY:                                              *
*                                                                   *
* Date:                                                             *
*                                                                   *
* Author:                                                           *
*                                                                   *
* Change Id:                                                        *
*                                                                   *
* Release:                                                          *
*                                                                   *
* Description of change:                                            *
*                                                                   *
*********************************************************************/

#ifndef __fls1_loc_h
#define __fls1_loc_h

#define TESLA_HEADER_ID 0x0B

#define PPM_ID "PPM"

#define FLASH_FLASHID_LIST "flashid.cfg"

#define FLASH_DEV_LIST "dev_list.fps"

#define FLASH_MODELS_LIST "models.cfg"

#define FLASH_ALGOCODE_LIST "algocode.cfg"

#define FLS_DCT3_EXTENSION_ID "TIA"

#define FLS1_ALIAS_ID_TXT "ALIAS_ID="

#define FLS1_SIZE_ID_TXT "size "

#define FLS1_BOOTSTRAP_PARAM_OFFSET 9

/* number of supported flashids per algocode file */
#define FLS1_MAX_FLASHIDS 64

#define FLS1_ALGOPARAM_OFS 0x12
#define FLS1_ALGOENTRIES_OFS 0x16

#define FLASH_READWRITE_RESULT_OK 0xff

#define FLS1_NUMALGOENTRIES_OFS 0x12

#define FLS1_ALGOPARAM_FVAL1_OFS 1
#define FLS1_ALGOPARAM_FVAL2_OFS 4
#define FLS1_ALGOPARAM_FLASHID_OFS 8

#define FLS1_GETSTATUS_CMD 0x04
#define FLS1_MCURESET_CMD 0x07
#define FLS1_WRITE_CMD 0x0B
#define FLS1_READ_CMD 0x0C
#define FLS1_ERASE_CMD 0x00
#define FLS1_ERASEALL_CMD 0x08
#define FLS1_READCONFIG_CMD 0x03

#define FLS1_PHONERAM_ID_DCT3 0x90
#define FLS1_PROGRAMSTATUS_ID 0x91

#define FLS1_ALGOCODE_CMD 0x78	/* 01111000 */
#define FLS1_CMDCODE1_CMD 0xF2	/* 11110010 */
#define FLS1_CMDCODE2_CMD 0xF3	/* 11110011 */
#define FLS1_CMDCODE3_CMD 0x76	/* 01110110 */
#define FLS1_CMDCODE4_CMD 0x70	/* 01110000 */

typedef struct {
	UINT8 U8cmd;
	UINT8 U8startaddr[3];
	UINT8 U8datachecksum;
	UINT8 U8count[3];
	UINT8 U8checksum;
} tFLS1_READWRITEPARAMS;

typedef struct {
	UINT8 U8cmd;
	UINT8 U8startaddr[3];
	UINT8 U8endaddr[3];
	UINT8 U8checksum;
} tFLS1_ERASEPARAMS;

#define FLASH_MCU_START_ADDRESS 0x200000
#define FLASH_MCU_ABSOLUTE_END 0x800000

#define FLASH_ERASE_BLOCK_SIZE 0x10000
//#define FLASH_ERASE_BLOCK_SIZE 0x1000

#define FLASH_PPM_ID "\xFF\xFFPPM\x00"
#define FLASH_PMM_ID "PMM"
#define FLASH_EEPROM_ID "EEPROM"

#define FLASH_MAX_NMP_LEN 5

#define FLASH_MCU2_BASE_OFFSET 0x3C
#define FLASH_LOADER_PARAM_OFFSET 14
#define FLASH_LOADER_PARAM_CFG_VAL1_OFFSET 1
#define FLASH_LOADER_PARAM_CFG_VAL2_OFFSET 4
#define FLASH_LOADER_PARAM_CFG_FLASHID_OFFSET 8
#define FLASH_MCU_PARAMBLOCK_OFFSET 60
#define FLASH_MCU_ORIGINAL_CHECKSUM1_OFFSET 34
#define FLASH_MCU_ORIGINAL_CHECKSUM2_OFFSET 56
#define FLASH_MCU_CHECKSUM_OFFSET1 36
#define FLASH_MCU_CHECKSUM_OFFSET2 39
#define FLASH_FVAL1_OFFSET 1
#define FLASH_FVAL2_OFFSET 4
#define FLASH_PPM_START_OFFSET 4	/* after end of MCU there is the PPM id */
#define FLASH_PPM_LANGUAGE_PACK_OFFSET 40
#define FLASH_PPM_LANGUAGE_PACK_PPM_DATA_OFFSET 44
#define FLASH_NEXT_PPM_OFFSET 20
#define FLASH_PPM_BLOCK_CHECKSUM_OFFSET 0
#define FLASH_PPM_BLOCK_SIZE_OFFSET 4
#define FLASH_PPM_BLOCK_HEADER_SIZE 4

#define FLASH_EEPROM_CHECKSUM_START 0x26
#define FLASH_EEPROM_CHECKSUM_OFFSET1 0x144
#define FLASH_EEPROM_CHECKSUM_OFFSET2 0x27A
#define FLASH_EEPROM_ENDSUM 0x1FE

#define FLASH_EEPROM_NSE5_CHECKSUM_OFFSET1 0x64
#define FLASH_EEPROM_NSE5_CHECKSUM_OFFSET2 0x18A
#define FLASH_EEPROM_NSE5_ENDSUM 0x0

#define FLASH_EEPROM_IMEI_OFFSET 0x32
#define FLASH_EEPROM_SECIMEI_OFFSET 0x162
#define FLASH_EEPROM_SECCODE_OFFSET 0x136

#define FLASH_PMM_SECCODE_ENTRY_ID 0x4A
#define FLASH_PMM_SECCODE_COUNTER_ENTRY_ID 0x48

#define FLASH_FIRST_PMM_OFFSET 0x20

#define FLASH_BLOCK_ID 0xF0F0FFF8
#define FLASH_BLOCK_ID_EEPROM 0x01
#define FLASH_BLOCK_ID_PMM 0x02
#define FLASH_BLOCK_HEADER_SIZE (4+2)
#define FLASH_PMM_BLOCK_ID_OFFSET 0x10

#define FLASH_PMM_ENABLED 0xF4
#define FLASH_PMM_DISABLED 0xA4
#define FLASH_PMM_BLOCKSTART_ID 0x55FF
#define FLASH_PMM_END_ID 0xFFFF

typedef struct {
	UINT8 U8enablecode;
	UINT8 U8blocktype;
	UINT16 U16blockidx;
	UINT16 U16pmmbeginstructure;
	UINT16 U16datachecksum;
	UINT16 U16datalength;
	UINT16 U16nextoffset;
} tFLASH_PMMENTRY;

typedef struct {
	char cfgdir[SYS_MAX_PATHNAME_LEN];
	char loaderdir[SYS_MAX_PATHNAME_LEN];
} tFLASH_DIRECTORIES;

typedef struct {
	UINT16 U16baseport;
	UINT8 U8fval1;
	UINT8 U8fval2;
	UINT32 U32flashendaddr;
	UINT32 U32safeend;
	UINT32 U32endofppm;
	BOOL Bemulate;
	BOOL Bskiperase;
	BOOL Bbinaryfileemulate;
	BOOL Bsafeflashing;
	UINT8 U8forcedflashsize;
	char emulatedflashfilename[SYS_MAX_PATHNAME_LEN];
	tFLS1_CONFIGURATIONRESPONSE_DCT3 config;
	tFLASH_DIRECTORIES directories;
	tFLASH_STATUS status;
	char flashid_list[SYS_MAX_FILENAME_LEN];
	char dct3_extension_id[SYS_MAX_EXTENSION_LEN];
	char aliasid_list[SYS_MAX_FILENAME_LEN];
	char models_list[SYS_MAX_FILENAME_LEN];
	char algocode_list[SYS_MAX_FILENAME_LEN];
	char NMP[FLASH_NMP_LEN];
} tFLS1_CONFIG;

typedef struct {
	UINT32 U32comdelay;
} tFLASH_DELAY;

#define FLS1_FCHK_START_OFFSET 0x40
#define FLS1_FCHK_SKIP_COUNT 0x20

#pragma pack(1)

typedef struct {
	UINT8 U8headerid;	/* must be 0x0B */
	UINT8 U8startaddr[3];	/* 24 bit */
	UINT8 U8fblockchecksum;
	UINT8 U8blocklen[3];	/* 24 bit */
	UINT8 U8headerchecksum;
} tWINTESLA_HEADER;

#define FLASH_TEMP_FLASH_FILE "$$$.FLS"

#define MAX_WINTESLA_BLOCK_SIZE 66000

#endif
