/********************************************************************
*                                                                   *
* FILE NAME:                                                        *
*                                                                   *
* PURPOSE:                                                          *
*                                                                   *
* FILE REFERENCES:                                                  *
*                                                                   *
* Name:                                                             *
*                                                                   *
* I/O:                                                              *
*                                                                   *
* Description:                                                      *
*                                                                   *
* EXTERNAL VARIABLES:                                               *
*                                                                   *
* Source:  <      >                                                 *
*                                                                   *
* Name:                                                             *
*                                                                   *
* Type:                                                             *
*                                                                   *
* I/O:                                                              *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* EXTERNAL REFERENCES:                                              *
*                                                                   *
* Name:                                                             *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* ABNORMAL TERMINATION CONDITIONS, ERROR AND WARNING MESSAGES:      *
*                                                                   *
* ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:                           *
*                                                                   *
* NOTES:                                                            *
*                                                                   *
* REQUIREMENTS/FUNCTIONAL SPECIFICATIONS REFERENCES:                *
*                                                                   *
* DEVELOPMENT HISTORY:                                              *
*                                                                   *
* Date:                                                             *
*                                                                   *
* Author:                                                           *
*                                                                   *
* Change Id:                                                        *
*                                                                   *
* Release:                                                          *
*                                                                   *
* Description of change:                                            *
*                                                                   *
********************************************************************/
#include "../include/config.h"

#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>

#include "../include/sysdef.h"
#include "../include/fls1pub.h"
#include "miscpub.h"
#include "mbusloc.h"
#include "mbuspub.h"

static BOOL MBUS_decodeMSID(tMSID * MSID);

extern eFLASH_ERROR flash_errno;

static UINT8 AU8readbuffer[4096];
static UINT8 AU8transferpacket[256];
static tMBUS_SIMLOCKSINFO mbus_simlocksinfo;

static UINT32 MBUSdcb[7] = {
	sizeof(DCB),
	9600,
	0x00001403,
	0x01000000,
	0x01080100,
	0x3F131100,
	0x0000001A
};

static tMBUS_CONFIG mbus_config;
static HANDLE hComm = NULL;

const tMBUSCMD MBUS_getversioncmd =
    { "Get version", 0x40, "\x00\x01\xC8\x10", 4, 11 };
const tMBUSCMD MBUS_getproductcodecmd =
    { "Get product code", 0x40, "\x00\x01\xCa\x01", 4, 11 };
const tMBUSCMD MBUS_enableextendedcmdscmd =
    { "Enable extended commands", 0x40, "\x00\x01\x64\x01", 4, 11 };
const tMBUSCMD MBUS_resetphonesetcmd =
    { "Reset phone settings", 0x40, "\x00\x01\x65\x01", 4, 11 };
const tMBUSCMD MBUS_resetuifsetcmd =
    { "Reset UIF settings", 0x40, "\x00\x01\x65\x08", 4, 11 };
const tMBUSCMD MBUS_resetallsetcmd =
    { "Reset all (UIF,SCM) settings", 0x40, "\x00\x01\x65\x38", 4, 11 };
const tMBUSCMD MBUS_resetdevsetcmd =
    { "Reset DEV settings", 0x40, "\x00\x01\x65\x02", 4, 11 };
const tMBUSCMD MBUS_resetfactorysetcmd =
    { "Reset full factory settings", 0x40, "\x00\x01\x65\xFF", 4, 11 };
const tMBUSCMD MBUS_enableservicemodecmd =
    { "Enable service mode", 0x40, "\x00\x01\x64\x02", 4, 11 };
const tMBUSCMD MBUS_resetphonecmd =
    { "Reset phone", 0x40, "\x00\x01\x64\x03", 4, 11 };
const tMBUSCMD MBUS_getIMEIcmd =
    { "Get IMEI", 0x40, "\x00\x01\x66", 3, 10 };
tMBUSCMD MBUS_setIMEIcmd =
    { "Set IMEI", 0x40,
"\x00\x01\x67\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 3 + 15,
10 };
tMBUSCMD MBUS_getMSIDcmd =
    { "Get MSID", 0x40, "\x00\x01" MBUS_GET_MSID_CMD_STR "\x00\x00",
4 + 0 + 1, 11 };
tMBUSCMD MBUS_setFAIDcmd =
    { "Set FAID", 0x40,
"\x00\x01" MBUS_SET_FAID_CMD_STR
"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 4 + 12 + 1, 11 };
tMBUSCMD MBUS_setSPlockFAIDcmd =
    { "Set SPlock", 0x40,
"\x00\x01" MBUS_SET_SPLOCK_CMD_STR
"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
4 + 12 + 12 + 1, 11 };
tMBUSCMD MBUS_setIMEIFAIDcmd =
    { "Set IMEI", 0x40,
"\x00\x01" MBUS_SET_IMEI_CMD_STR
"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
4 + 12 + 8 + 1, 11 };
const tMBUSCMD MBUS_getMCUchecksum =
    { "Get MCU checksum", 0x40, "\x00\x01\xc8\x02", 4, 11 };
const tMBUSCMD MBUS_getoriginalIMEIcmd =
    { "Get original IMEI", 0x40, "\x00\x01\xCC\x01", 4, 11 };
tMBUSCMD MBUS_setoriginalIMEIcmd =
    { "Set original IMEI", 0x40,
"\x00\x01\xCD\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 4 + 15, 11 };
const tMBUSCMD MBUS_getASICcmd =
    { "Get ASIC", 0x40, "\x00\x01\xC8\x0C", 4, 11 };
const tMBUSCMD MBUS_getCOBBAcmd =
    { "Get COBBA", 0x40, "\x00\x01\xC8\x0D", 4, 11 };
const tMBUSCMD MBUS_getinternalDSPcmd =
    { "Get internal DSP sw", 0x40, "\x00\x01\xC8\x09", 4, 11 };
const tMBUSCMD MBUS_getexternalDSPcmd =
    { "Get external DSP sw", 0x40, "\x00\x01\xC8\x03", 4, 11 };
const tMBUSCMD MBUS_getlanguagepackcmd =
    { "Get language pack", 0x40, "\x00\x01\xC8\x12", 4, 11 };
const tMBUSCMD MBUS_getsecuritycodecmd =
    { "Get security code", 0x40, "\x00\x01\x6e\x01", 4, 11 };
const tMBUSCMD MBUS_getsimlockinfocmd =
    { "Get simlock info", 0x40, "\x00\x01\x8a\x00", 4, 11 };
tMBUSCMD MBUS_setsimlockinfocmd =
    { "Set simlock info", 0x40, "\x00\x01\x82\x01\0x00",
5 + sizeof(tWRITESIMLOCKINFO) + 1, 11 };
tMBUSCMD MBUS_opensimlockcmd =
    { "Open simlock", 0x40, "\x00\x01\x81\x01\x00", 5, 11 };
const tMBUSCMD MBUS_preopensimlockcmd =
    { "Preopen simlock", 0x40, "\x00\x01\x02\x03\x1f\x11\x01\x01\x10\x00",
10, 11 };
const tMBUSCMD MBUS_getordernumbercmd =
    { "Get order number", 0x40, "\x00\x01\xca\x02", 4, 11 };
const tMBUSCMD MBUS_getproductsncmd =
    { "Get product sn", 0x40, "\x00\x01\xca\x03", 4, 11 };
const tMBUSCMD MBUS_getbasicproductcodecmd =
    { "Get basic product code", 0x40, "\x00\x01\xca\x04", 4, 11 };
const tMBUSCMD MBUS_getmanufacturemonthcmd =
    { "Get manufacture month", 0x40, "\x00\x01\xcc\x02", 4, 11 };
const tMBUSCMD MBUS_getpurchasedatecmd =
    { "Get purchase date", 0x40, "\x00\x01\xcc\x04", 4, 11 };
const tMBUSCMD MBUS_getprofilecmd =
    { "Get profile", 0x40, "\x00\x01\x6a", 3, 9 };
tMBUSCMD MBUS_setprofilecmd =
    { "Set profile", 0x40, "\x00\x01\x6b\x00\x00\x00\x00", 3 + 4, 9 };


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_OpenCOMPort(UINT8 U8comport, UINT32 U32comdelay)
{
	char COMport[5];

	mbus_config.Binitialized = FALSE;

	MDBGSTR2("MBUS_OpenCOMPort : opening COM%d with delay %d",
		 U8comport, U32comdelay);

	if (U8comport < 1 || U8comport > 4) {
		flash_errno = FE_INVALID_COM_PORT;
		return FALSE;
	}

	if (hComm) {		/* opened? */
		CloseHandle(hComm);
	}
	sprintf(COMport, "COM%i", U8comport);
	hComm =
	    CreateFile(COMport, GENERIC_READ | GENERIC_WRITE, 0, NULL,
		       OPEN_EXISTING,
		       FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, NULL);
	if (hComm == NULL) {
		flash_errno = FE_INVALID_COM_PORT;
		return FALSE;
	}

	if (!SetCommState(hComm, (DCB *) & MBUSdcb)) {
		CloseHandle(hComm);
		flash_errno = FE_INVALID_COM_PORT;
		return FALSE;
	}

	GetCommTimeouts(hComm, &mbus_config.newcommtimeouts);
	mbus_config.oldcommtimeouts = mbus_config.newcommtimeouts;
	mbus_config.newcommtimeouts.ReadTotalTimeoutConstant =
	    mbus_config.U16comdelay = (UINT8) U32comdelay;
	SetCommTimeouts(hComm, &mbus_config.newcommtimeouts);

	mbus_config.U8sequencenumber = 0x00;
	mbus_config.Binitialized = TRUE;
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void MBUS_CloseCOMPort(void)
{
	if (mbus_config.Binitialized) {
		HDBGSTR("MBUS_CloseCOMPort : Closing COM port");
		SetCommTimeouts(hComm, &mbus_config.oldcommtimeouts);
		CloseHandle(hComm);
	}
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_checkreply(UINT8 * pU8packet, UINT8 * pU8reply,
			    UINT32 U32replylen)
{
	UINT16 U16packetsize, U16replysize;
	SINT16 count;
	UINT8 U8replychk;
	UINT8 U8replylen;

	if (pU8packet[0] != pU8reply[0])
		return FALSE;

	U16packetsize = misc_swapword(*(UINT16 *) (pU8packet + 1));
	U16replysize = *(UINT16 *) (pU8reply + 1);
	if (U16packetsize != U16replysize)
		return FALSE;

	U8replylen = pU8reply[5];

	if ((UINT32) U8replylen + 8 != U32replylen)
		return FALSE;

	for (count = 0, U8replychk = 0; count < U8replylen + 7; count++)
		U8replychk ^= pU8reply[count];

	if (pU8reply[U8replylen + 7] != U8replychk)
		return FALSE;

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void MBUS_createack(UINT8 * pU8packet, UINT8 * request,
			   UINT32 * U32acklen)
{
	SINT16 count;
	UINT8 U8current = 4, U8packetlen = pU8packet[5];

	request[0] = pU8packet[0];
	request[1] = pU8packet[2];
	request[2] = pU8packet[1];
	request[3] = FBUS_FRTYPE_ACK;

	request[U8current++] = pU8packet[6 + U8packetlen];	/* sequence number */

	/* Checksum */
	request[U8current] = 0;
	for (count = 0; count < U8current; count++)
		request[U8current] ^= request[count];

	*U32acklen = (UINT32) U8current;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static UINT32 MBUS_createframe(UINT8 U8seqnum, UINT16 message_length,
			       UINT8 U8message_type, UINT8 * buffer,
			       UINT8 * out_buffer)
{
	SINT16 count, current = 0;
	UINT8 checksum;

	if (message_length > MBUS_MAX_CONTENT_LENGTH)
		return FALSE;

	/* Now construct the message header. */

	out_buffer[current++] = MBUS_FRAME_ID;	/* Start of the frame indicator */
	out_buffer[current++] = FBUS_DEVICE_PHONE;	/* Destination */
	out_buffer[current++] = MBUS_DEVICE_PC1;	/* Source */
	out_buffer[current++] = U8message_type;	/* Type */
	out_buffer[current++] = message_length / 256;	/* Length1+1 (seq num) */
	out_buffer[current++] = message_length % 256;	/* Length2+1 (seq num) */

	/* Copy in data if any. */

	if (message_length != 0) {
		memcpy(out_buffer + current, buffer, message_length);
		current += message_length;
		out_buffer[current] = U8seqnum;	/* copy sequence number */
		current++;
	}

	/* Now calculate checksum over entire message and append to message. */

	/* All bytes */
	checksum = 0;
	for (count = 0; count < current; count++)
		checksum ^= out_buffer[count];

	out_buffer[current++] = checksum;

	return current;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_sendcommand(UINT8 * pU8packet, UINT32 U32packetlen,
			     BOOL Bneedreply, UINT32 * pU32replysize)
{
	static UINT8 ack[6];
	UINT32 U32acklen;
	SINT32 i;
	OVERLAPPED osWrite, osRead;
	DWORD dwErrorFlags;
	COMSTAT ComStat;
#ifdef DUMP_MBUS_REPLY
	char numbuff[12];
	char linebuff[80];
#endif

	if (pU32replysize)
		*pU32replysize = 0;

	memset(AU8readbuffer, 0, sizeof(AU8readbuffer));

	if (!WriteFile
	    (hComm, pU8packet, U32packetlen, &mbus_config.U32writtenbytes,
	     &osWrite)) {
		MDBGSTR("MBUS_sendcommand : cannot write to COM port");
		/* not so keen on handling comm error */
		ClearCommError(hComm, &dwErrorFlags, &ComStat);
		return FALSE;
	}

	if (!ReadFile
	    (hComm, AU8readbuffer, sizeof(AU8readbuffer),
	     &mbus_config.U32receivedbytes, &osRead)) {
		MDBGSTR("MBUS_sendcommand : cannot read from COM port");
		/* not so keen on handling comm error */
		ClearCommError(hComm, &dwErrorFlags, &ComStat);
		return FALSE;
	}

	if (mbus_config.U32receivedbytes == 0) {
		MDBGSTR
		    ("MBUS_sendcommand : phone does not respond to command");
		return FALSE;
	}

	MBUS_createack(pU8packet, ack, &U32acklen);

#ifdef DUMP_MBUS_REPLY
	strcpy(linebuff, "");
	for (i = 0; i < mbus_config.U32receivedbytes; i++) {
		if (i % 16) {
			MDBGSTR(linebuff);
			strcpy(linebuff, "");
		}
		sprintf(numbuff, "%02X", AU8readbuffer);
		strcat(linebuff, numbuff);
	}
#endif

	mbus_config.S32replyoffset =
	    misc_memstr(AU8readbuffer, mbus_config.U32receivedbytes, ack,
			U32acklen);
	if (mbus_config.S32replyoffset == -1) {
		HDBGSTR
		    ("MBUS_sendcommand : invalid reply to command from phone");
		return FALSE;
	}

	if (!Bneedreply)	/* If we don't need reply then don't check it */
		return TRUE;

	mbus_config.S32replyoffset += U32acklen;

	if (mbus_config.U32receivedbytes < 8) {
		HDBGSTR("MBUS_sendcommand : reply is too short");
		return FALSE;
	}

	for (i = mbus_config.S32replyoffset;
	     i < (SINT32) (mbus_config.U32receivedbytes - 8); i++) {
		if (MBUS_checkreply
		    (pU8packet, AU8readbuffer + i,
		     mbus_config.U32receivedbytes - i)) {
			*pU32replysize = mbus_config.U32receivedbytes;
			MBUS_createack(AU8readbuffer + i, ack, &U32acklen);

			if (!WriteFile
			    (hComm, ack, sizeof(ack),
			     &mbus_config.U32writtenbytes, NULL)) {
				MDBGSTR
				    ("MBUS_sendcommand : cannot write to COM port");
				/* not so keen on handling comm error */
				ClearCommError(hComm, &dwErrorFlags,
					       &ComStat);
				return FALSE;
			}

			if (!ReadFile
			    (hComm, ack, sizeof(ack),
			     &mbus_config.U32receivedbytes, NULL)) {
				MDBGSTR
				    ("MBUS_sendcommand : cannot read from COM port");
				/* not so keen on handling comm error */
				ClearCommError(hComm, &dwErrorFlags,
					       &ComStat);
				return FALSE;
			}

			mbus_config.S32replyoffset = i;
			return TRUE;
		}
	}

	return FALSE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_cmd(UINT8 * pU8seqnum, UINT8 U8messagetype, char *pU8cmd,
	      UINT32 U32cmdlen, BOOL Bneedreply, UINT32 * pU32replysize)
{
	UINT32 U32retry = 0;
	UINT32 U32packetlen;
	BOOL Bfinished = FALSE;
	if (U32cmdlen > 0x1000)	/*hm, safe max */
		return FALSE;

	(*pU8seqnum)++;

	U32packetlen =
	    MBUS_createframe(*pU8seqnum, (UINT16) U32cmdlen, U8messagetype,
			     (UINT8 *) pU8cmd, AU8transferpacket);
	MDBGSTR1("MBUS_cmd : size of command is %d", U32packetlen);

	while (!Bfinished) {
		PurgeComm(hComm,
			  PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR |
			  PURGE_RXCLEAR);
		Bfinished =
		    MBUS_sendcommand(AU8transferpacket, U32packetlen,
				     Bneedreply, pU32replysize);
		if (!Bfinished) {
			if (++U32retry == RETRY_TIMES) {
				HDBGSTR1
				    ("MBUS_cmd : timed out after %d tries",
				     U32retry);
				return FALSE;
			}
		}
	}

	PurgeComm(hComm,
		  PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR |
		  PURGE_RXCLEAR);
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SecurityCommand(const tMBUSCMD * mbuscmd, UINT8 * pU8reply,
			  UINT32 U32replybuffersize,
			  UINT32 * pU32replysize)
{
	memset(pU8reply, 0, U32replybuffersize);
	if (!MBUS_cmd
	    (&mbus_config.U8sequencenumber, mbuscmd->U8messagetype,
	     (char *) mbuscmd->AU8frame, mbuscmd->U8framesize, TRUE,
	     pU32replysize)) {
		flash_errno = FE_CANT_EXECUTE_MBUS_CMD;
		LDBGSTR1("MBUS_SecurityCommand : cannot execute '%s'",
			 mbuscmd->AU8name);
		return FALSE;
	}

	*pU32replysize -=
	    (mbus_config.S32replyoffset + mbuscmd->U8replyshift);
	memcpy(pU8reply,
	       AU8readbuffer + mbus_config.S32replyoffset +
	       mbuscmd->U8replyshift, *pU32replysize);

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SetServiceMode(void)
{
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];
	UINT32 U32replysize;

	if (!MBUS_SecurityCommand
	    (&MBUS_enableextendedcmdscmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		MDBGSTR
		    ("MBUS_SetServiceMode : cannot enable extended commands");
		flash_errno = FE_CANT_EXECUTE_MBUS_CMD;
		return FALSE;
	}

	mbus_config.U8phoneid = AU8buffer[0];

	if (!MBUS_SecurityCommand
	    (&MBUS_enableservicemodecmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		MDBGSTR
		    ("MBUS_SetServiceMode : cannot enable service mode");
		flash_errno = FE_CANT_EXECUTE_MBUS_CMD;
		return FALSE;
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_GetSimlockInfo(void)
{
	tREADSIMLOCKINFO *simlockinfo;
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	UINT32 idx, i, j, k;
	UINT8 U8flag, U8char;
#ifdef SAVE_SIMLOCKDATA
	FILE *rfile;
#endif

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_GetSimLockInfo : MBUS is not initialized");
		return FALSE;
	}

	memset(&mbus_simlocksinfo, 0, sizeof(mbus_simlocksinfo));

	simlockinfo = (tREADSIMLOCKINFO *) & AU8buffer[0];
	if (!MBUS_SecurityCommand
	    (&MBUS_getsimlockinfocmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}
#ifdef SAVE_SIMLOCKDATA
	rfile = fopen("simlock.in", "wb");
	if (rfile) {
		fwrite(AU8buffer, 1, sizeof(tREADSIMLOCKINFO) + 1, rfile);
		fclose(rfile);
	}
#endif

	for (i = 0; i < MBUS_MAX_SIMLOCKS; i++) {
		idx = MBUS_SIMLOCKINDEX[i] - 1;
		mbus_simlocksinfo.simlockinfo[idx].Bopen =
		    !(simlockinfo->U8locksclosed & (1 << idx));
		mbus_simlocksinfo.simlockinfo[idx].Buser =
		    simlockinfo->U8lockstype & (1 << idx);

		U8flag = (MBUS_SIMLOCKPOS[i] & 0x01 ? 0x0F : 0xF0);
		for (j = 0, k = MBUS_SIMLOCKPOS[i] >> 1;
		     j < (UINT32) MBUS_SIMLOCKLEN[i]; j++) {
			U8char = simlockinfo->U8locksinfo[k];
			if (U8flag == 0xF0) {
				mbus_simlocksinfo.simlockinfo[idx].
				    value[j] =
				    misc_chartohexchar((UINT8)
						       ((U8char & U8flag)
							>> 4));
				U8flag = 0x0F;
			} else {
				mbus_simlocksinfo.simlockinfo[idx].
				    value[j] =
				    misc_chartohexchar((UINT8)
						       (U8char & U8flag));
				U8flag = 0xF0;
				k++;
			}
		}
	}
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_SetSimlockInfo(UINT8 U8lockidx)
{
	tWRITESIMLOCKINFO *simlockinfo;
	UINT8 AU8outbuffer[1024], AU8buffer[1024];
	UINT32 U32replysize;
	UINT32 idx, i, j, k;
	UINT8 U8flag, U8char;
#ifdef SAVE_SIMLOCKDATA
	FILE *rfile;
#endif

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_SetSimLockInfo : MBUS is not initialized");
		return FALSE;
	}

	memset(AU8buffer, 0, sizeof(AU8buffer));

	simlockinfo = (tWRITESIMLOCKINFO *) & AU8buffer[0];

	for (i = 0; i < MBUS_MAX_SIMLOCKS; i++) {
		idx = MBUS_SIMLOCKINDEX[i] - 1;

		U8flag = (MBUS_SIMLOCKPOS[i] & 0x01 ? 0x0F : 0xF0);
		for (j = 0, k = MBUS_SIMLOCKPOS[i] >> 1;
		     j < (UINT32) MBUS_SIMLOCKLEN[i]; j++) {
			U8char =
			    misc_hexchartobyte(mbus_simlocksinfo.
					       simlockinfo[idx].value[j]);

			if (U8flag == 0xF0) {
				simlockinfo->U8locksinfo[k] |=
				    (U8char << 4);
				U8flag = 0x0F;
			} else {
				simlockinfo->U8locksinfo[k] |= U8char;
				U8flag = 0xF0;
				k++;
			}
		}
	}

#ifdef SAVE_SIMLOCKDATA
	rfile = fopen("simlock.out", "wb");
	if (rfile) {
		fwrite(AU8buffer, 1, sizeof(tWRITESIMLOCKINFO) + 1, rfile);
		fclose(rfile);
	}
#endif

	idx = U8lockidx;

	/* we've either changed the value of the lock or closed the lock */
	if (mbus_simlocksinfo.simlockinfo[idx].Bvaluediffers ||
	    (mbus_simlocksinfo.simlockinfo[idx].Bsetupdiffers
	     && !mbus_simlocksinfo.simlockinfo[idx].Bopen)) {
		MDBGSTR2
		    ("MBUS_SetSimLockInfo : closing simlock %d with %s",
		     idx + 1, mbus_simlocksinfo.simlockinfo[idx].value);

		MBUS_setsimlockinfocmd.AU8frame[4] = (1 << idx);
		memcpy(&MBUS_setsimlockinfocmd.AU8frame[5], AU8buffer,
		       sizeof(tWRITESIMLOCKINFO));

		if (!MBUS_SecurityCommand
		    (&MBUS_setsimlockinfocmd, AU8outbuffer,
		     sizeof(AU8outbuffer), &U32replysize)) {
			return FALSE;
		}
	}

	if (mbus_simlocksinfo.simlockinfo[idx].Bopen) {
		MDBGSTR("MBUS_SetSimLockInfo : preopening simlock");
		if (!MBUS_SecurityCommand
		    (&MBUS_preopensimlockcmd, AU8outbuffer,
		     sizeof(AU8outbuffer), &U32replysize)) {
			return FALSE;
		}

		MDBGSTR1("MBUS_SetSimLockInfo : opening simlock %d",
			 idx + 1);
		MBUS_opensimlockcmd.AU8frame[4] = (1 << idx);
		if (!MBUS_SecurityCommand
		    (&MBUS_opensimlockcmd, AU8outbuffer,
		     sizeof(AU8outbuffer), &U32replysize)) {
			return FALSE;
		}
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void MBUS_SetupSecurityCommand(tMBUSCMD * mbuscmd, UINT8 * pAU8args)
{
	mbuscmd->AU8frame[3] = mbus_config.U8phoneid;
	if ((SINT16) mbuscmd->U8framesize - 3 > 0) {
		if (pAU8args) {
			memcpy(&mbuscmd->AU8frame[4], pAU8args,
			       mbuscmd->U8framesize - 3);
		}
	}

	mbuscmd->AU8frame[mbuscmd->U8framesize - 1] =
	    misc_calculate8bitchecksumas16bit(&mbuscmd->AU8frame[2],
					      mbuscmd->U8framesize - 3);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SaveSecurityPhoneInfo(char *logfile)
{
	FILE *rfile;
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	UINT32 i;
	tMSID decodedMSID;

	if (!mbus_config.Binitialized) {
		HDBGSTR
		    ("MBUS_SaveSecurityPhoneInfo : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		return FALSE;
	}

	misc_checkfilename(logfile);
	rfile = fopen(logfile, "wt");
	if (!rfile) {
		flash_errno = FE_CANT_CREATE_FILE;
		return FALSE;
	}

	fprintf(rfile, "SECURITY PHONE INFO by KNOK (build %d)\n\n",
		ENGINE_BUILD);

	if (!MBUS_SecurityCommand
	    (&MBUS_getversioncmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Software version: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getproductcodecmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Product code: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getlanguagepackcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Language pack: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getprofilecmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Profile: %02X %02X %02X %02X\n", AU8buffer[0],
		AU8buffer[1], AU8buffer[2], AU8buffer[3]);

	if (!MBUS_SecurityCommand
	    (&MBUS_getIMEIcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "IMEI: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getoriginalIMEIcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Original IMEI: %s\n", AU8buffer);

	MBUS_SetupSecurityCommand(&MBUS_getMSIDcmd, NULL);
	if (!MBUS_SecurityCommand
	    (&MBUS_getMSIDcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	if (U32replysize == MBUS_MSID_LEN) {
		fprintf(rfile, "MSID: ");

		for (i = 0; i < U32replysize - 3; i++)
			fprintf(rfile, "%02X", AU8buffer[i]);

		fprintf(rfile, "\n");
	}

	memcpy(&decodedMSID, AU8buffer, sizeof(decodedMSID));
	if (!MBUS_decodeMSID(&decodedMSID)) {
		fprintf(rfile, ">>> MSID is invalid <<<\n");
	} else {
		fprintf(rfile, "MSID id: %02X\n", decodedMSID.U8id);
		fprintf(rfile, "MSID signature: %02X%02X%02X\n",
			decodedMSID.U8sig[0], decodedMSID.U8sig[1],
			decodedMSID.U8sig[2]);
		fprintf(rfile, "FCHK: %08X\n",
			misc_swapdword(decodedMSID.U32fchk));
		fprintf(rfile, "Cobba ID: %08X %02X\n",
			misc_swapdword(decodedMSID.U32cobba),
			decodedMSID.U8cobba);
	}

	if (!MBUS_SecurityCommand
	    (&MBUS_getMCUchecksum, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "MCU checksum: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getASICcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "ASIC: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getCOBBAcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "COBBA: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getinternalDSPcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "DSP internal sw: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getexternalDSPcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "DSP external sw: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getsecuritycodecmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Security code: %s\n", AU8buffer);

	if (!MBUS_GetSimlockInfo()) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Simlock info:\n");

	for (i = 0; i < MBUS_MAX_SIMLOCKS; i++) {
		fprintf(rfile, "Simlock %d is a %s lock and it's %s (",
			i + 1,
			(mbus_simlocksinfo.simlockinfo[i].
			 Buser ? "user" : "factory"),
			(mbus_simlocksinfo.simlockinfo[i].
			 Bopen ? "open" : "closed"));
		fprintf(rfile, "%s)\n",
			mbus_simlocksinfo.simlockinfo[i].value);
	}

	if (!MBUS_SecurityCommand
	    (&MBUS_getordernumbercmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Order number: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getproductsncmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Product serial number: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getbasicproductcodecmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Basic product code: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getmanufacturemonthcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Manufacture month: %s\n", AU8buffer);

	if (!MBUS_SecurityCommand
	    (&MBUS_getpurchasedatecmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		fclose(rfile);
		return FALSE;
	}

	fprintf(rfile, "Purchase date: %s\n",
		(strlen((char *) AU8buffer) ==
		 0 ? "Not available" : (char *) AU8buffer));

	fclose(rfile);

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_GetVersion(UINT8 * pAU8buffer, UINT32 U32bufferlen)
{
	UINT32 U32versionsize;

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_GetVersion : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		MDBGSTR("MBUS_GetVersion : cannot set service mode");
		return FALSE;
	}

	if (!MBUS_SecurityCommand
	    (&MBUS_getversioncmd, pAU8buffer, U32bufferlen,
	     &U32versionsize)) {
		MDBGSTR("MBUS_GetVersion : cannot send command");
		return FALSE;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_ResetGivenSettings(const tMBUSCMD * mbuscmd,
				    UINT8 U8multiplier)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	BOOL Bsuccess;

	if (U8multiplier > 0) {
		mbus_config.newcommtimeouts.ReadTotalTimeoutConstant =
		    U8multiplier * mbus_config.U16comdelay;
		SetCommTimeouts(hComm, &mbus_config.newcommtimeouts);
	}

	Bsuccess =
	    MBUS_SecurityCommand(mbuscmd, AU8buffer, sizeof(AU8buffer),
				 &U32replysize);

	if (U8multiplier > 0) {
		mbus_config.newcommtimeouts.ReadTotalTimeoutConstant =
		    mbus_config.U16comdelay;
		SetCommTimeouts(hComm, &mbus_config.newcommtimeouts);
	}

	if (!Bsuccess) {
		LDBGSTR1("MBUS_ResetGivenSettings : cannot apply '%s'",
			 mbuscmd->AU8name);
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_ResetSettings(eMBUS_RESET_TYPE reset_type)
{
	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_ResetSettings : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		return FALSE;
	}

	if (reset_type == MBUS_RESET_TYPE_PHONE
	    || reset_type == MBUS_RESET_TYPE_ALL) {
		if (!MBUS_ResetGivenSettings(&MBUS_resetphonesetcmd, 0)) {
			return FALSE;
		}
	}

	if (reset_type == MBUS_RESET_TYPE_UIF
	    || reset_type == MBUS_RESET_TYPE_ALL) {
		if (!MBUS_ResetGivenSettings(&MBUS_resetuifsetcmd, 2)) {
			return FALSE;
		}
	}

	if (reset_type == MBUS_RESET_TYPE_SCM
	    || reset_type == MBUS_RESET_TYPE_ALL) {
		if (!MBUS_ResetGivenSettings(&MBUS_resetallsetcmd, 2)) {
			return FALSE;
		}
	}

	if (reset_type == MBUS_RESET_TYPE_DEV
	    || reset_type == MBUS_RESET_TYPE_ALL) {
		if (!MBUS_ResetGivenSettings(&MBUS_resetdevsetcmd, 0)) {
			return FALSE;
		}
	}

	if (reset_type == MBUS_RESET_TYPE_FACTORY
	    || reset_type == MBUS_RESET_TYPE_ALL) {
		if (!MBUS_ResetGivenSettings(&MBUS_resetfactorysetcmd, 5)) {
			return FALSE;
		}
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_ResetPhone(void)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	BOOL Bsuccess;

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_ResetPhone : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		return FALSE;
	}

	Bsuccess =
	    MBUS_SecurityCommand(&MBUS_resetphonecmd, AU8buffer,
				 sizeof(AU8buffer), &U32replysize);

	if (!Bsuccess) {
		LDBGSTR("MBUS_ResetPhone : reset failed");
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SetSimlock(UINT8 U8lockidx, char *val, BOOL Bopen)
{
	UINT8 U8len, i;
	BOOL Bfound, Bval, Bset, Bsuccess;

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_SetSimlock : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		return FALSE;
	}

	for (i = 0, Bfound = 0; i < MBUS_MAX_SIMLOCKS && !Bfound; i++) {
		Bfound = MBUS_SIMLOCKINDEX[i] == U8lockidx;
		if (Bfound) {
			U8len = MBUS_SIMLOCKLEN[i];
		}
	}

	if (!Bfound) {
		LDBGSTR("MBUS_SetSimlock : Invalid lock number");
		return FALSE;
	}

	if (!MBUS_GetSimlockInfo()) {
		return FALSE;
	}

	Bval = (val != NULL && strlen(val) > 0 && val[0] != '.');
	Bset = (Bopen == TRUE || Bopen == FALSE);

	if (!Bval && !Bset)	// nothing is set ;(
	{
		return TRUE;
	}

	if (Bval) {
		if (Bset) {
			HDBGSTR3
			    ("MBUS_SetSimlock : setting simlock %d with %s as %s",
			     U8lockidx, val, (Bopen ? "open" : "closed"));
		} else {
			HDBGSTR2
			    ("MBUS_SetSimlock : setting simlock %d with %s",
			     U8lockidx, val);
		}
	} else {
		HDBGSTR2("MBUS_SetSimlock : setting simlock %d as %s",
			 U8lockidx, (Bopen ? "open" : "closed"));
	}

	U8lockidx--;

	if (Bset) {
		HDBGSTR1("MBUS_SetSimlock : old lock setup is %s",
			 (mbus_simlocksinfo.simlockinfo[U8lockidx].
			  Bopen ? "open" : "closed"));
		mbus_simlocksinfo.simlockinfo[U8lockidx].Bsetupdiffers =
		    Bopen !=
		    mbus_simlocksinfo.simlockinfo[U8lockidx].Bopen;
		mbus_simlocksinfo.simlockinfo[U8lockidx].Bopen = Bopen;
	} else {
		mbus_simlocksinfo.simlockinfo[U8lockidx].Bsetupdiffers =
		    FALSE;
	}

	if (Bval) {
		HDBGSTR1("MBUS_SetSimlock : old lock value is %s",
			 mbus_simlocksinfo.simlockinfo[U8lockidx].value);
		mbus_simlocksinfo.simlockinfo[U8lockidx].Bvaluediffers =
		    strncmp(mbus_simlocksinfo.simlockinfo[U8lockidx].value,
			    val, U8len) != 0;
		memset(mbus_simlocksinfo.simlockinfo[U8lockidx].value, 0,
		       sizeof(mbus_simlocksinfo.simlockinfo[U8lockidx].
			      value));
		strncpy(mbus_simlocksinfo.simlockinfo[U8lockidx].value,
			val, U8len);
	} else {
		mbus_simlocksinfo.simlockinfo[U8lockidx].Bvaluediffers =
		    FALSE;
	}

	mbus_config.newcommtimeouts.ReadTotalTimeoutConstant =
	    2 * mbus_config.U16comdelay;
	SetCommTimeouts(hComm, &mbus_config.newcommtimeouts);

	Bsuccess = MBUS_SetSimlockInfo(U8lockidx);

	mbus_config.newcommtimeouts.ReadTotalTimeoutConstant =
	    mbus_config.U16comdelay;
	SetCommTimeouts(hComm, &mbus_config.newcommtimeouts);

	return Bsuccess;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SetContrast(UINT8 U8val)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	BOOL Bsuccess;
	UINT8 U8profile[4];

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_SetContrast : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		return FALSE;
	}

	Bsuccess =
	    MBUS_SecurityCommand(&MBUS_getprofilecmd, AU8buffer,
				 sizeof(AU8buffer), &U32replysize);
	if (!Bsuccess) {
		LDBGSTR("MBUS_SetContrast : cannot get profile");
		return FALSE;
	}

	memcpy(U8profile, AU8buffer, sizeof(U8profile));

	MDBGSTR4("MBUS_SetContrast : old profile is %02X %02X %02X %02X",
		 U8profile[0], U8profile[1], U8profile[2], U8profile[3]);

	U8profile[3] &= (128 + 64 + 32);	// clear last 5 bit
	U8profile[3] |= (U8val & 31);

	MDBGSTR4("MBUS_SetContrast : new profile is %02X %02X %02X %02X",
		 U8profile[0], U8profile[1], U8profile[2], U8profile[3]);

	memcpy(&MBUS_setprofilecmd.AU8frame[3], U8profile,
	       sizeof(U8profile));

	Bsuccess =
	    MBUS_SecurityCommand(&MBUS_setprofilecmd, AU8buffer,
				 sizeof(AU8buffer), &U32replysize);
	if (!Bsuccess) {
		LDBGSTR("MBUS_SetContrast : cannot set profile");
		return FALSE;
	}

	Bsuccess =
	    MBUS_SecurityCommand(&MBUS_resetphonecmd, AU8buffer,
				 sizeof(AU8buffer), &U32replysize);
	if (!Bsuccess) {
		LDBGSTR("MBUS_SetContrast : reset failed");
	}

	return Bsuccess;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static UINT32 getMSIDsig(UINT8 U8MSIDid)
{
	if (U8MSIDid < LOWEST_SUPPORTED_MSID
	    || U8MSIDid > HIGHEST_SUPPORTED_MSID) {
		return 0;
	}
	return U32MSIDsig[(U8MSIDid - LOWEST_SUPPORTED_MSID)];
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static const tMSIDhash *getMSIDhashtable(UINT8 U8MSIDid,
					 const tMSIDhash phashbase[])
{
	if (U8MSIDid < LOWEST_SUPPORTED_MSID
	    || U8MSIDid > HIGHEST_SUPPORTED_MSID) {
		return NULL;
	}
	return &phashbase[(U8MSIDid - LOWEST_SUPPORTED_MSID)];
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void createMSIDdecodetable(UINT8 * ptr)
{
	UINT8 U8code;
	UINT8 i;
	UINT8 *start = ptr;
	tMSIDhash tmphash;

	U8code = *ptr;
	tmphash[0x1] = *ptr;
	tmphash[0x9] = *ptr;
	tmphash[0x4] = *ptr;
	tmphash[0x8] = *ptr;
	tmphash[0xB] = *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x5] = *ptr;
	tmphash[0x9] ^= *ptr;
	tmphash[0x8] ^= *ptr;
	tmphash[0x6] = *ptr;
	tmphash[0xA] = *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x1] ^= *ptr;
	tmphash[0x3] = *ptr;
	tmphash[0xB] ^= *ptr;
	tmphash[0x6] ^= *ptr;
	tmphash[0xA] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x0] = *ptr;
	tmphash[0x8] ^= *ptr;
	tmphash[0x7] = *ptr;
	tmphash[0xB] ^= *ptr;
	tmphash[0xA] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x1] ^= *ptr;
	tmphash[0x5] ^= *ptr;
	tmphash[0x0] ^= *ptr;
	tmphash[0x8] ^= *ptr;
	tmphash[0x3] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x1] ^= *ptr;
	tmphash[0x9] ^= *ptr;
	tmphash[0x0] ^= *ptr;
	tmphash[0x2] = *ptr;
	tmphash[0xA] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x5] ^= *ptr;
	tmphash[0x3] ^= *ptr;
	tmphash[0x7] ^= *ptr;
	tmphash[0x2] ^= *ptr;
	tmphash[0xA] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x0] ^= *ptr;
	tmphash[0x4] ^= *ptr;
	tmphash[0x3] ^= *ptr;
	tmphash[0xB] ^= *ptr;
	tmphash[0x2] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x5] ^= *ptr;
	tmphash[0x9] ^= *ptr;
	tmphash[0x0] ^= *ptr;
	tmphash[0x4] ^= *ptr;
	tmphash[0x7] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x1] ^= *ptr;
	tmphash[0x5] ^= *ptr;
	tmphash[0x4] ^= *ptr;
	tmphash[0x2] ^= *ptr;
	tmphash[0x6] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x9] ^= *ptr;
	tmphash[0x7] ^= *ptr;
	tmphash[0xB] ^= *ptr;
	tmphash[0x2] ^= *ptr;
	tmphash[0x6] ^= *ptr;

	ptr++;
	U8code ^= *ptr;
	tmphash[0x4] ^= *ptr;
	tmphash[0x8] ^= *ptr;
	tmphash[0x3] ^= *ptr;
	tmphash[0x7] ^= *ptr;
	tmphash[0x6] ^= *ptr;

	for (i = 0; i < sizeof(tMSIDhash); i++)
		tmphash[i] ^= U8code;

	memcpy(start, tmphash, sizeof(tmphash));
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static UINT8 reversebyte(UINT8 pU8byte)
{
	UINT8 U8tmp = pU8byte;
	UINT8 U8new = 0;
	UINT8 i;

	for (i = 0; i < 8; i++) {
		if (U8tmp & (1 << (7 - i))) {
			U8new |= (1 << i);
		}
	}

	return U8new;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void reversebits(UINT8 * ptr)
{
	UINT8 i;
	UINT8 tmphash[MSID_TABLE_SIZE];

	memcpy(tmphash, ptr, sizeof(tmphash));

	for (i = 0; i < sizeof(tmphash); i++) {
		ptr[MSID_TABLE_SIZE - i - 1] = reversebyte(tmphash[i]);
	}
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void createcode(UINT8 ** ptr1, UINT8 ** ptr2, UINT8 * l, UINT8 * m)
{
	*l ^= (**ptr2);
	(*ptr2)++;
	*m ^= (**ptr2);
	(*ptr2)++;

	(**ptr1) ^= *l;
	(*ptr1)++;
	(**ptr1) ^= *m;
	(*ptr1)++;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void createcodefromstart(UINT8 ** ptr1, UINT8 ** ptr2, UINT8 * l,
				UINT8 * m)
{
	*l = *m = 0;
	createcode(ptr1, ptr2, l, m);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void prepareMSIDdecode(UINT8 seedvalue, UINT8 * streamptr,
			      UINT8 * tableptr)
{
	UINT8 l, m;
	UINT8 *ptr1, *ptr2;

	ptr1 = streamptr;
	ptr2 = tableptr;
	createcodefromstart(&ptr1, &ptr2, &l, &m);
	l = m = seedvalue;
	createcode(&ptr1, &ptr2, &l, &m);
	createcodefromstart(&ptr1, &ptr2, &l, &m);
	createcodefromstart(&ptr1, &ptr2, &l, &m);

	l = m = seedvalue;
	createcode(&ptr1, &ptr2, &l, &m);
	createcodefromstart(&ptr1, &ptr2, &l, &m);

	createMSIDdecodetable(streamptr);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void rotateMSIDntimes(UINT8 * ptr, UINT8 ntimes)
{
	UINT8 i, U8prevbit, U8bit;

	ptr += 3;

	for (i = 0; i < ntimes; i++) {
		U8bit = (*ptr & 1);
		U8prevbit = U8bit;

		ptr -= 3;

		U8bit = (*ptr & 1);
		*ptr >>= 1;
		*ptr |= (U8prevbit << 7);
		U8prevbit = U8bit;

		ptr++;

		U8bit = (*ptr & 1);
		*ptr >>= 1;
		*ptr |= (U8prevbit << 7);
		U8prevbit = U8bit;

		ptr++;

		U8bit = (*ptr & 1);
		*ptr >>= 1;
		*ptr |= (U8prevbit << 7);
		U8prevbit = U8bit;

		ptr++;

		*ptr >>= 1;
		*ptr |= (U8prevbit << 7);
	}
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void rotateMSID(UINT8 * ptr1, UINT8 * ptr2)
{
	rotateMSIDntimes(ptr1, 0x0a);
	rotateMSIDntimes(ptr2, 0x1f);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void alterMSID(UINT8 * streamptr)
{
	UINT8 l, i, *m, *k, *j;
	UINT8 tmphash[MSID_TABLE_SIZE];

	memcpy(tmphash, streamptr, MSID_TABLE_SIZE);
	m = &streamptr[MSID_TABLE_SIZE];
	k = &tmphash[7];
	j = &tmphash[3];

	for (i = 0; i < MSID_TABLE_SIZE; i++) {
		l = ~(*k) | (*j);

		m--;

		if (k == &tmphash[0]) {
			k = &tmphash[MSID_TABLE_SIZE];
		}

		k--;

		if (j == &tmphash[0]) {
			j = &tmphash[MSID_TABLE_SIZE];
		}

		j--;

		*m ^= l;
	}
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void cryptMSID(tMSID * pMSID, const tMSIDhash seed, tMSIDhash hash)
{
	UINT8 i;
	UINT8 *ptr, *pmsid = (UINT8 *) & pMSID->U32fchk;

	ptr = (UINT8 *) pMSID->U8sig;
	for (i = 0; i < sizeof(tMSIDhash) - 1; i++) {
		prepareMSIDdecode(seed[i], pmsid, (UINT8 *) hash);
		rotateMSID(pmsid, ptr);
		alterMSID(pmsid);
		rotateMSID(ptr, pmsid);
	}

	prepareMSIDdecode(seed[sizeof(tMSIDhash) - 1], pmsid,
			  (UINT8 *) hash);
	reversebits(pmsid);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_decodeMSID(tMSID * MSID)
{
	const tMSIDhash *pMSIDhash;
	tMSIDhash MSIDhash;
	UINT32 U32sig;
	UINT32 U32msidsig;

	pMSIDhash = getMSIDhashtable(MSID->U8id, MSIDdecodehash);
	if (pMSIDhash == NULL) {
		LDBGSTR1("MBUS_decodeMSID : invalid MSID type %02X",
			 MSID->U8id);
		return FALSE;
	}

	memcpy(&MSIDhash, pMSIDhash, sizeof(MSIDhash));

	createMSIDdecodetable(MSIDhash);
	reversebits(MSIDhash);

	cryptMSID(MSID, MSIDdecodeseeds, MSIDhash);
	U32sig = getMSIDsig(MSID->U8id);

	U32msidsig = 0;
	memcpy(&U32msidsig, MSID->U8sig, sizeof(MSID->U8sig));
	if (U32sig != U32msidsig) {
		LDBGSTR2
		    ("MBUS_decodeMSID : invalid signature in decoded MSID (%06X/%06X)",
		     U32sig, U32msidsig);
		return FALSE;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL MBUS_SetupFAIDCalculation(const tMSIDhash phashbase[],
				      tMSID * pMSID, tMSIDhash * MSIDhash,
				      UINT32 * pU32fchk)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	UINT8 i, U8MSIDid;
	const tMSIDhash *pMSIDhash;
	tMSID MSID;
	UINT8 AU8cobba[sizeof(UINT32)];

	if (!mbus_config.Binitialized) {
		HDBGSTR("MBUS_SetFAID : MBUS is not initialized");
		return FALSE;
	}

	if (!MBUS_SetServiceMode()) {
		return FALSE;
	}

	MBUS_SetupSecurityCommand(&MBUS_getMSIDcmd, NULL);
	if (!MBUS_SecurityCommand
	    (&MBUS_getMSIDcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}

	memcpy(&MSID, AU8buffer, sizeof(MSID));
	if (!MBUS_decodeMSID(&MSID)) {
		return FALSE;
	}

	U8MSIDid = MSID.U8id;
	if (pU32fchk) {
		*pU32fchk = MSID.U32fchk;
	}

	if (U8MSIDid < LOWEST_SUPPORTED_MSID
	    || U8MSIDid > HIGHEST_SUPPORTED_MSID) {
		LDBGSTR1
		    ("MBUS_SetupFAIDCalculation : invalid MSID type %02X",
		     MSID.U8id);
		return FALSE;
	}

	pMSIDhash = getMSIDhashtable(U8MSIDid, phashbase);
	if (pMSIDhash == NULL) {
		LDBGSTR1
		    ("MBUS_SetupFAIDCalculation : invalid MSID type %02X",
		     MSID.U8id);
		return FALSE;
	}

	memcpy(MSIDhash, pMSIDhash, sizeof(*MSIDhash));
	memcpy(AU8cobba, &MSID.U32cobba, sizeof(AU8cobba));
	for (i = 0; i < sizeof(AU8cobba); i++) {
		(*MSIDhash)[i] ^= AU8cobba[i];
	}

	memset(pMSID, 0, sizeof(*pMSID));
	pMSID->U8id = U8MSIDid;

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SetFAID(UINT32 U32fchk)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	tMSIDhash MSIDhash;
	UINT32 U32originalfchk;
	tMSID MSID;
	UINT8 U8fchk[sizeof(UINT32)];

	if (!MBUS_SetupFAIDCalculation
	    (MSIDencodehash, &MSID, &MSIDhash, &U32originalfchk)) {
		return FALSE;
	}

	if (U32fchk == 0) {
		U32fchk = U32originalfchk;
		HDBGSTR1("MBUS_SetFAID : using " UINT32_FORMAT
			 " as checksum", U32fchk);
		memcpy(U8fchk, &U32fchk, sizeof(U8fchk));
		U8fchk[0] ^= 0xE8;
		U8fchk[1] ^= 0x35;
		U8fchk[2] ^= 0x9F;
		U8fchk[3] ^= 0x76;
		memcpy(&U32fchk, U8fchk, sizeof(U32fchk));
		U32fchk = ~U32fchk;
	}

	MSID.U32fchk = U32fchk;
	MSID.U32cobba = ~U32fchk;

	cryptMSID(&MSID, MSIDencodeseeds, MSIDhash);

	MBUS_SetupSecurityCommand(&MBUS_setFAIDcmd,
				  (UINT8 *) & MSID.U32fchk);
	if (!MBUS_SecurityCommand
	    (&MBUS_setFAIDcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}

	LDBGSTR("MBUS_SetFAID : MSID is updated succesfully");

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void MBUS_BCDtonumber(UINT8 * pAU8in, UINT16 U16inlen, UINT8 * pout)
{
	UINT8 i, j;
	UINT8 U8digit;
	UINT32 U32num;

	for (i = 0; i < U16inlen / 2; i++) {
		for (j = 0, U32num = 0; j < 2; j++, pAU8in++) {
			U8digit = misc_hexchartobyte(*pAU8in);
			U32num <<= 4;
			U32num |= U8digit;
		}

		*pout = (UINT8) U32num;
		pout++;
	}
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_ResetSPlock(void)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	tMSIDhash MSIDhash;
	tMSID MSID;

	if (!MBUS_SetupFAIDCalculation
	    (SPLockencodehash, &MSID, &MSIDhash, NULL)) {
		return FALSE;
	}

	memcpy(&MSID.U32fchk, MBUS_DEFAULT_SPLOCK_KEY, MSID_TABLE_SIZE);
	cryptMSID(&MSID, MSIDencodeseeds, MSIDhash);

	memset(AU8buffer, 0, sizeof(AU8buffer));
	memcpy(AU8buffer, &MSID.U32fchk, MSID_TABLE_SIZE);	/* save first part of ID */

	memcpy(&MSID.U32fchk, MBUS_DEFAULT_SPLOCK_VALUE, MSID_TABLE_SIZE);
	cryptMSID(&MSID, MSIDencodeseeds, MSIDhash);

	memcpy(&AU8buffer[MSID_TABLE_SIZE], &MSID.U32fchk, MSID_TABLE_SIZE);	/* save next 12 bytes */

	MBUS_SetupSecurityCommand(&MBUS_setSPlockFAIDcmd, AU8buffer);
	if (!MBUS_SecurityCommand
	    (&MBUS_setSPlockFAIDcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}

	LDBGSTR("MBUS_ResetSPlock : SPlock is cleared");

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL MBUS_SetIMEI(UINT8 * newIMEI)
{
	UINT8 AU8buffer[1024];
	UINT32 U32replysize;
	tMSIDhash MSIDhash;
	tMSID MSID;
	UINT8 i, U8IMEI[MSID_TABLE_SIZE], U8newIMEIbin[32];

	if (!MBUS_SetupFAIDCalculation
	    (IMEIencodehash, &MSID, &MSIDhash, NULL)) {
		return FALSE;
	}

	if (strlen((char *) newIMEI) != MBUS_IMEI_LEN) {
		HDBGSTR
		    ("MBUS_SetIMEI : invalid length of new IMEI, trying to use original");
		if (!MBUS_SecurityCommand
		    (&MBUS_getoriginalIMEIcmd, AU8buffer,
		     sizeof(AU8buffer), &U32replysize)) {
			LDBGSTR
			    ("MBUS_SetIMEI : cannot read original IMEI");
			return FALSE;
		}

		strncpy(newIMEI, AU8buffer, 2 * MBUS_IMEI_LEN);
		newIMEI[2 * MBUS_IMEI_LEN] = 0;

		MDBGSTR1("MBUS_SetIMEI : using %s as IMEI", newIMEI);
		if (strlen((char *) newIMEI) != MBUS_IMEI_LEN) {
			LDBGSTR
			    ("MBUS_SetIMEI : invalid length of original IMEI");
			return FALSE;
		}
	}

	MDBGSTR1("MBUS_SetIMEI : setting up IMEI as %s", newIMEI);

	memset(U8IMEI, 0, sizeof(U8IMEI));
	memcpy(U8IMEI, MBUS_DEFAULT_IMEI_ID, sizeof(MBUS_DEFAULT_IMEI_ID));
	memcpy(&U8IMEI[9], MBUS_DEFAULT_IMEI_KEY,
	       sizeof(MBUS_DEFAULT_IMEI_KEY));

	memset(U8newIMEIbin, 0, sizeof(U8newIMEIbin));
	MBUS_BCDtonumber(newIMEI, (MBUS_IMEI_LEN - 1), U8newIMEIbin);	/* make 7 bytes from 15 (skip last checksum) */
	memcpy(&U8IMEI[2], U8newIMEIbin, (MBUS_IMEI_LEN - 1) / 2);

	memcpy(&MSID.U32fchk, U8IMEI, MSID_TABLE_SIZE);
	cryptMSID(&MSID, MSIDencodeseeds, MSIDhash);

	memset(AU8buffer, 0, sizeof(AU8buffer));
	memcpy(AU8buffer, &MSID.U32fchk, MSID_TABLE_SIZE);	/* store ID first */

	U8newIMEIbin[7] = 0xFF;	/* don't ask me why */
	memcpy(&AU8buffer[MSID_TABLE_SIZE], U8newIMEIbin, (MBUS_IMEI_LEN - 1) / 2 + 1);	/* then new IMEI */

	MBUS_SetupSecurityCommand(&MBUS_setIMEIFAIDcmd, AU8buffer);
	if (!MBUS_SecurityCommand
	    (&MBUS_setIMEIFAIDcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}

	memset(&MBUS_setoriginalIMEIcmd.AU8frame[4], 0, MBUS_IMEI_LEN);
	memset(&MBUS_setIMEIcmd.AU8frame[4], 0, MBUS_IMEI_LEN);
	for (i = 0; i < MBUS_IMEI_LEN + 1; i++) {
		MBUS_setoriginalIMEIcmd.AU8frame[4 + i] =
		    newIMEI[i] & 0x0f;
		MBUS_setIMEIcmd.AU8frame[4 + i] = newIMEI[i] & 0x0f;
	}

	if (!MBUS_SecurityCommand
	    (&MBUS_setoriginalIMEIcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}

	if (!MBUS_SecurityCommand
	    (&MBUS_setIMEIcmd, AU8buffer, sizeof(AU8buffer),
	     &U32replysize)) {
		return FALSE;
	}

	LDBGSTR("MBUS_SetIMEI : new IMEI is set");

	return TRUE;
}
