
/********************************************************************
* NAME:                                                             *
*                                                                   *
* PURPOSE:                                                          *
*                                                                   *
* GLOBAL VARIABLES:                                                 *
*                                                                   *
* Variable:                                                         *
*                                                                   *
* Type:                                                             *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* DEVELOPMENT HISTORY:                                              *
*                                                                   *
* Date:                                                             *
*                                                                   *
* Author:                                                           *
*                                                                   *
* Change Id:                                                        *
*                                                                   *
* Release:                                                          *
*                                                                   *
* Description of change:                                            *
*                                                                   *
*********************************************************************/

#ifndef __mbus_loc_h
#define __mbus_loc_h

#define RETRY_TIMES 10

#define FBUS_DEVICE_PHONE   0x00	/* Nokia mobile phone. */

#define FBUS_FRTYPE_ACK     0x7f	/* Acknowledge of the received frame. */

/* This byte is at the beginning of all GSM Frames sent over MBUS to Nokia
   phones. */
#define MBUS_FRAME_ID       0x1f

/* Our PC (MBUS). */
#define MBUS_DEVICE_PC1     0x10

#define MBUS_MAX_CONTENT_LENGTH  (1000)

#define MBUS_MSID_LEN 16
#define MBUS_GET_MSID_CMD 0xb4
#define MBUS_GET_MSID_CMD_STR "\xb4"
#define MBUS_SET_FAID_CMD 0xb8
#define MBUS_SET_FAID_CMD_STR "\xb8"
#define MBUS_SET_IMEI_CMD 0xb6
#define MBUS_SET_IMEI_CMD_STR "\xb6"
#define MBUS_SET_SPLOCK_CMD 0xba
#define MBUS_SET_SPLOCK_CMD_STR "\xba"

typedef struct {
	UINT8 U8lockstype;
	UINT8 U8locksclosed;
	UINT16 U16dummy;
	UINT8 U8locksinfo[12];
} tREADSIMLOCKINFO;

typedef struct {
	UINT16 U16dummy;
	UINT8 U8locksinfo[12];
} tWRITESIMLOCKINFO;

#define MBUS_MAX_SIMLOCKS 4
static const UINT8 MBUS_SIMLOCKLEN[] = { 5, 10, 4, 4 };
static const UINT8 MBUS_SIMLOCKPOS[] = { 0, 5, 16, 20 };
static const UINT8 MBUS_SIMLOCKINDEX[] = { 1, 4, 2, 3 };

#define MBUS_MAX_SIMLOCKLEN 12

typedef struct {
	BOOL Bopen;
	BOOL Buser;
	BOOL Bvaluediffers;
	BOOL Bsetupdiffers;
	char value[MBUS_MAX_SIMLOCKLEN];
} tMBUS_SIMLOCKINFO;

typedef struct {
	tMBUS_SIMLOCKINFO simlockinfo[MBUS_MAX_SIMLOCKS];
} tMBUS_SIMLOCKSINFO;

typedef struct {
	COMMTIMEOUTS newcommtimeouts, oldcommtimeouts;
	UINT32 U32writtenbytes, U32receivedbytes;
	SINT32 S32replyoffset;
	UINT8 U8sequencenumber;
	UINT8 U8comport;
	UINT8 U8phoneid;
	BOOL Binitialized;
	UINT16 U16comdelay;
} tMBUS_CONFIG;

#define MSID_TABLE_SIZE 12
#define LOWEST_SUPPORTED_MSID 0x81
#define HIGHEST_SUPPORTED_MSID 0x83

#pragma pack(1)

typedef struct {
	UINT8 U8id;
	UINT32 U32fchk;
	UINT32 U32cobba;
	UINT8 U8sig[3];		// 24bit signature
	UINT8 U8cobba;
} tMSID;

typedef UINT8 tMSIDhash[MSID_TABLE_SIZE];

static const UINT32 U32MSIDsig[] = { 0xA7A6A5, 0xAAA9A8, 0xABADAC };

static const tMSIDhash MSIDdecodeseeds =
    { 0xD0, 0x16, 0x2C, 0x58, 0xB0, 0x71, 0xE2, 0xD5, 0x5A, 0x67, 0xCE,
0x8D };

static const tMSIDhash MSIDdecodehash[] = {
	{0x8a, 0xb2, 0x23, 0x9c, 0x2f, 0xf9, 0x54, 0xe1, 0xfa, 0xd7, 0x66,
	 0xdc},
	{0x57, 0xf4, 0xb2, 0x41, 0x27, 0xf4, 0xe4, 0xea, 0x55, 0xcf, 0xfe,
	 0xa0},
	{0x5d, 0x04, 0xe0, 0xdf, 0xbd, 0x16, 0x25, 0x77, 0xf6, 0x2e, 0xd8,
	 0x58}
};

static const tMSIDhash MSIDencodeseeds =
    { 0xB1, 0x73, 0xE6, 0x5A, 0xAB, 0x47, 0x8E, 0x0D, 0x1A, 0x34, 0x68,
0x0B };

static const tMSIDhash MSIDencodehash[] = {
	{0x2c, 0x03, 0x6f, 0x34, 0x5d, 0x23, 0xc6, 0x58, 0x03, 0x09, 0x52,
	 0xc4},
	{0x4f, 0xb9, 0x6c, 0x27, 0x02, 0xDD, 0x77, 0xDD, 0x06, 0xE3, 0xE2,
	 0xDF},
	{0xB8, 0x33, 0x80, 0x04, 0xB8, 0x48, 0x85, 0x1A, 0x2D, 0xCE, 0xA1,
	 0x28}
};

static const tMSIDhash IMEIencodehash[] = {
	{0xCA, 0x63, 0x5A, 0xE7, 0x19, 0xFA, 0xA6, 0x4C, 0x0D, 0x78, 0xFC,
	 0x16},
	{0x0B, 0xAE, 0xFC, 0x8C, 0xC7, 0xFC, 0x79, 0x2A, 0xC1, 0x94, 0xC2,
	 0x37},
	{0xCA, 0x63, 0x5A, 0xE7, 0x19, 0xFA, 0xA6, 0x4C, 0x0D, 0x78, 0xFC,
	 0x16}
};

static const tMSIDhash SPLockencodehash[] = {
	{0x7B, 0xB4, 0xD0, 0xEF, 0x9E, 0xB2, 0x0A, 0xBE, 0x73, 0xDA, 0xD3,
	 0x35},
	{0xD1, 0xA2, 0x5F, 0xEB, 0x4F, 0xF0, 0x2D, 0x7B, 0x0F, 0x4C, 0xE1,
	 0xC3},
	{0x7B, 0xB4, 0xD0, 0xEF, 0x9E, 0xB2, 0x0A, 0xBE, 0x73, 0xDA, 0xD3,
	 0x35}
};

#define MBUS_DEFAULT_IMEI_ID "\x79\x29"
#define MBUS_DEFAULT_IMEI_KEY "\x00\x00\x00"

#define MBUS_DEFAULT_SPLOCK_KEY "\x00\x10\x10\x00\x00\x00\x00\x10\x00\x79\x54\xc2"

#define MBUS_DEFAULT_SPLOCK_VALUE "\x00\x00\x00\x00\x29\x00\x00\x00\x00\x00\x54\xc2"

#define MBUS_IMEI_LEN 15

#endif
