/**
 * DCT3 generic code uploader -- www.blacksphere.tk
 * blacksphere@goliath.darktech.org
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/io.h>
#include "../include/config.h"
#include "../include/sysdef.h"
#include "../include/fls1pub.h"

#ifndef WIN32

#endif

/** handle -uploadcode command **/
static BOOL cmd_uploadcode(char *NMP, char *path) {

  UINT8 U8fval1,U8fval2;
  char bloadername[SYS_MAX_FILENAME_LEN];
  tFLS1_CONFIGURATIONRESPONSE_DCT3 configuration;
  tFLS1_DELAYBLOCK delayblock;
  FILE *rfile;
  size_t codesize;
  UINT8 *code;
  //UINT8 testcode[] = {0x00, 0x00, 0x00, 0x05, 0xE7, 0xFE}; /* jump self */
  UINT8 *d;
  int pktime;
	FLS1_GetFlashParameters(NMP, &U8fval1, &U8fval2, bloadername);
	if(bloadername[0] == 'x') {
		printf("Boot loader configuration not found!\n");
		printf("Make sure this program is invoked in the directory with loaders/ and config/\n");
		return;
	}
	printf("%s bloader='%s'\n",NMP,bloadername);
	//strcpy(bloadername, "bootloader.bin");

	if (!FLS1_SendBootstrapDCT3(bloadername,U8fval1,50,&configuration)) {
	    printf("Sending bootstrap failed!\n");
            return FALSE;
	}
	printf("Bootstrap sent ASIC ID = %02x.\n",configuration.U8asicid);
	d = (UINT8*)&configuration.flashid[0];
	printf("Bootstrap sent FLASHID = %02x%02x%02x%02x.\n",d[0],d[1],d[2],d[3]);
	FLS1_GetDefaultTimes(&delayblock);
	printf("Loading code %s from disk.\n", path);
	/*
	typedef BOOL (*pSendCodeDCT3)(UINT8 *code,
				size_t codesize,
				tFLS1_DELAYBLOCK *delayblock);
	*/
	/*
	load code from disk, todo 
	*/
	
	rfile = fopen(path,"rb");
	if (!rfile)
	{
		printf("Could not open code file\n");
		return FALSE;
	}
	fseek(rfile,0,SEEK_END);
	codesize = ftell(rfile);
	fseek(rfile,0,SEEK_SET);
	code = malloc(codesize);
	fread(code, 1, codesize, rfile);
	fclose(rfile);

	printf("Loaded %i bytes.\n",codesize);


	if(!FLS1_SendCodeDCT3(code, codesize, &delayblock)) {
	    printf("Sending code failed!\n");
            return FALSE;
	}
	/*
	if(!FLS1SendCodeDCT3(testcode, sizeof(testcode), &delayblock)) {
	    printf("Sending code failed!\n");
            return FALSE;
	}
	*/
	printf("Code sent.\n");
	free(code);
	
	//usleep(30000);
	//FLS1_WaitForEdge(0, 100);
	pktime = misc_gettickcount();
	while(1) {
		unsigned char buffer[1];
		if(FLS1_WaitForEdge(1, 100)) {
			int length;
			FLS1_ReceiveFromPhone(buffer, 1, 0, &delayblock);
			length = buffer[0];			
			if(length == 0xFF)
				/* exit */
				break;
			printf("Packet: %i Length %02X\n", misc_gettickcount() - pktime, length);
			/* TODO: look up packet indebug list */
			while(length > 0) {
				FLS1_ReceiveFromPhone(buffer, 1, 0, &delayblock);
				printf("%02X ", buffer[0]);
				length--;
			}
			printf("\n");
			pktime = misc_gettickcount();
		}
	}
}

void cmd_mylog(char *logstr,BOOL Biserror,BOOL Bshowaserror)
{
  printf("%s\n",logstr);
  if (Biserror && Bshowaserror)
    printf("ERRNO : %08X\n",FLS1_GetErrno());
}


int main(int argc, char **argv) {
	if(argc!=3) {
		fprintf(stderr, "Usage: %s <phone type(NHM-5)> <file>\n", argv[0]);
		exit(1);
	}
	char configpath[SYS_MAX_PATHNAME_LEN]="config";
	char bloaderpath[SYS_MAX_PATHNAME_LEN]="loaders";
  	char emulatefilename[SYS_MAX_FILENAME_LEN]="";
  	char modelslist[SYS_MAX_FILENAME_LEN]="models.cfg";
  	char algocodelist[SYS_MAX_FILENAME_LEN]="algocode.cfg";
  	char flashidlist[SYS_MAX_FILENAME_LEN]="flashid.cfg";
  	char devlist[SYS_MAX_FILENAME_LEN]="dev_list.fps";
  	char logfilename[SYS_MAX_FILENAME_LEN]="";
  	char tiaext[SYS_MAX_EXTENSION_LEN]="tia";

  	UINT8 U8lpt = 1; /* LPT1 */
  	UINT8 U8com = 1; /* COM1 */
  	UINT32 U32comdelay=100;
  	//UINT8 U8numcommands;
  	//BOOL Bneedfchk = FALSE;
  	//BOOL Bneedflasher = FALSE;
  	//BOOL Bneedmbus = FALSE;
  	BOOL Bsafeflashing = FALSE;
  	eREDIRECTION_TYPE redirectiontype = REDIRECTION_TYPE_NONE;
  	char NMP[8];
  	
  	pLogCallout logcallout=NULL;
  	pGaugeInit gaugeinit=NULL;
  	pGaugeDone gaugedone=NULL;
  	pGaugeUpdate gaugeupdate=NULL;
  	UINT32 *pU32fchk=NULL;
  	int debuglvl = 666;

  	ioperm(0x378, 0x8, 1);
  	// realtimePriority();
	//cmd_mylog("Cannot initialize device",0,0);
	if (!FLS1_Initialize(debuglvl, gaugeinit, gaugedone, 
			     gaugeupdate, &cmd_mylog, redirectiontype, 
			     logfilename, FLASH_DELAY_TYPE_RDTSC, 0, 
			     U8lpt, U8com, U32comdelay, Bsafeflashing)) {
    		cmd_mylog("Cannot initialize device",0,0);
    		return;
  	}
  	
  	FLS1_SetDirectory(FLASH_DIRECTORY_LOADER,bloaderpath);
  	FLS1_SetDirectory(FLASH_DIRECTORY_CFG,configpath);
  	FLS1_SetFlashIDList(flashidlist);
  	FLS1_SetAlgoCodeList(algocodelist);
  	FLS1_SetAlgoCodeExtension(tiaext);
  	FLS1_SetAliasIDList(devlist);
  	FLS1_SetModelsList(modelslist);
  	
  	strncpy(NMP, argv[1], 8);
  	NMP[7] = 0;
	cmd_uploadcode(NMP, argv[2]);
	return 0;
}
